/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MsgsPerSecondDataProd.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.cewolf.dataproducers;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;

import de.laures.cewolf.DatasetProduceException;
import de.laures.cewolf.DatasetProducer;

public final class QueuesMessagesDataProd implements DatasetProducer {

    private static final int COLLECTOR_REFRESH_FREQUENCY = 10000;

    /**
     * 
     */
    private static final long serialVersionUID = -14245479763L;

    private static final String actionTitle = "show PEtALS messages per second repartition chart";

    final Logger log = Logger.getLogger(QueuesMessagesDataProd.class);

    public QueuesMessagesDataProd() {
        super();
    }

    /**
     * Returns the id of the producer
     * 
     * @return an id as a String
     */
    public String getProducerId() {
        return new String("QueuesMessagesProducer");
    }

    /**
     * Indicates if dataset produced has expired. method is called by the Cewolf
     * framework to check if a formerly produced data can be reused. As datas
     * are evolving and retrieved in an MBean, method always returns true to
     * force reloading datas.
     * 
     * @return true if the data which had been produced with the passed in
     *         parameters has expired since its creation, false otherwise
     */
    public boolean hasExpired(Map params, Date since) {
        return true;
    }

    @SuppressWarnings("unchecked")
    public Object produceDataset(Map params) throws DatasetProduceException {

        final String component = (String) params.get("component");
        final String server = (String) params.get("server");
        final long sinceTime = ((Long) params.get("sinceTime")).longValue();
        final long endTime = sinceTime - 5000 + (60000 * 5);

        final TimeSeriesCollection dataset = new TimeSeriesCollection();

        try {
            final CollectorServer collector = CollectorManager.getCollector();
            final Map<Long, Map<String, Long>> allQueues = collector.getCollectorClient(server)
                    .getTransportMonitoringService().getComponentQueuesStatus(server, component);

            final List<String> transporters = collector.getCollectorClient(server)
                    .getTransportMonitoringService().getTransporters();

            final List<String> formattedTransporters = new ArrayList<String>();

            for (final String name : transporters) {
                final String nameArray[] = name.split("-");
                String formattedQueue = "";
                if (nameArray.length > 1) {
                    formattedQueue = nameArray[nameArray.length - 1] + " queue";
                } else {
                    formattedQueue = name;
                }
                formattedTransporters.add(formattedQueue);
            }

            final Map<String, Map<Long, Long>> queueTimedatas = new HashMap<String, Map<Long, Long>>();

            /** build a Map of each component queues datas* */
            for (final Long time : allQueues.keySet()) {
                final long roundedTime = (time / QueuesMessagesDataProd.COLLECTOR_REFRESH_FREQUENCY)
                        * QueuesMessagesDataProd.COLLECTOR_REFRESH_FREQUENCY;
                for (final String queue : allQueues.get(time).keySet()) {
                    /** format queue name* */
                    final String nameArray[] = queue.split("-");
                    String formattedQueue = "";
                    if (nameArray.length > 1) {
                        formattedQueue = nameArray[nameArray.length - 1] + " queue";
                    } else {
                        formattedQueue = queue;
                    }

                    Map<Long, Long> ts = null;
                    if (queueTimedatas.containsKey(formattedQueue)) {
                        ts = queueTimedatas.get(formattedQueue);
                    } else {
                        ts = new HashMap<Long, Long>();
                    }
                    ts.put(roundedTime, allQueues.get(time).get(queue));
                    queueTimedatas.put(formattedQueue, ts);
                }
            }

            /** transform datas to datseries* */
            for (final String queue : formattedTransporters) {
                /**
                 * for each queue, build a new dataset in the correct time
                 * interval *
                 */
                final TimeSeries ts = new TimeSeries(queue, Second.class);
                for (long timestamp = (sinceTime / QueuesMessagesDataProd.COLLECTOR_REFRESH_FREQUENCY)
                        * QueuesMessagesDataProd.COLLECTOR_REFRESH_FREQUENCY; timestamp <= endTime; timestamp += QueuesMessagesDataProd.COLLECTOR_REFRESH_FREQUENCY) {
                    long valueToAdd = 0L;
                    if ((queueTimedatas != null) && (queueTimedatas.containsKey(queue))) {
                        final Map<Long, Long> values = queueTimedatas.get(queue);
                        if (values.containsKey(timestamp)) {
                            valueToAdd = values.get(timestamp);
                        }
                    }
                    ts.add(new Second(new Date(timestamp)), valueToAdd);
                }
                dataset.addSeries(ts);
            }
            /**
             * as the exception is managed by the chart, we have to log and
             * throw it*
             */
        } catch (final DataCollectorServerException e) {
            this.log.error("Can't " + QueuesMessagesDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (final DataCollectorClientException e) {
            this.log.error("Can't " + QueuesMessagesDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (DataCollectorClientDoesNotExistException e) {
            this.log.error("Can't " + QueuesMessagesDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.log.error("Can't " + QueuesMessagesDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        }

        return dataset;
    }

}
